/**
 * WildCAT: A Generic Framework for Context-Aware Applications.
 * Copyright (C) 2008 Bull S.A.S.
 * Copyright (C) 2008 EMN
 * Contact: wildcat@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 * Authors:
 *		- Pierre-Charles David (initial)
 *		- Nicolas Loriant
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */
package org.ow2.wildcat.hierarchy;

import org.testng.Assert;
import org.testng.annotations.Test;

/**
 * Unit test for {@link org.ow2.wildcat.hierarchy.Path}.
 */
public class TestPath {
    private String[] validPaths;
    private String[] invalidPaths;

    private String[] resourcePaths;
    private String[] attributePaths;

    private String[] absolutePaths;
    private String[] relativePaths;

    private String[] noncanonicalize;
    private String[] canonicalize;

    private String[][] stripTop;

    /**
     * Initialize paths for tests
     */
    public TestPath() {
        validPaths = new String[] { "self://proc/cpu#load", "/proc/cpu#load",
                "proc/cpu#load", "cpu#load", "#load", };
        invalidPaths = new String[] { "self://proc#error/cpu#load",
                "//proc/cpu#load", "cpu#error#load", };
        resourcePaths = new String[] { "self://proc/cpu", "/proc/cpu",
                "proc/cpu", "cpu", };
        attributePaths = new String[] { "self://proc/cpu#load",
                "/proc/cpu#load", "proc/cpu#load", "cpu#load", "#load", };
        absolutePaths = new String[] { "/proc/cpu#load", "/cpu#load", "/#load",
                "self://proc/cpu#load", "self://cpu#load", "self://#load", };
        relativePaths = new String[] { "proc/cpu#load", "cpu#load", "#load", };
        noncanonicalize = new String[] { "self://./../toto/../*/toto#toto",
                "self://*/*#*", ".././*/toto" };
        canonicalize = new String[] { "self://*/toto#toto", "self://*/*#*",
        "../*/toto" };
        stripTop = new String[][] {
                { "self://proc/cpus/cpu#load", "proc/cpus/cpu#load" },
                { "proc/cpus/cpu#load", "cpus/cpu#load" },
                { "cpus/cpu#load", "cpu#load" }, { "cpu#load", "#load" },
                { "#load", null }, };
    }

    /**
     * Test method for
     * {@link org.ow2.wildcat.hierarchy.Path#Path(java.lang.String)}.
     */
    @Test
    public void testPathStringValid() throws MalformedPathException {
        for (String s : this.validPaths) {
            new Path(s);
        }
    }

    /**
     * Test method for
     * {@link org.ow2.wildcat.hierarchy.Path#Path(java.lang.String)}.
     */
    @Test
    public void testPathStringInvalid() {
        for (String s : this.invalidPaths) {
            try {
                new Path(s);                
            } catch (MalformedPathException e) {
            	Assert.assertTrue(false);
            }
        }
    }

    /**
     * Test method for
     * {@link org.ow2.wildcat.hierarchy.Path#appendResource(java.lang.String)}.
     */
    @Test
    public void testAppendResource() {
        try {
            for (String s : this.resourcePaths) {
                Assert.assertNotNull(new Path(s).appendResource("toto"));
            }
            for (String s : this.attributePaths) {
                Assert.assertNull(new Path(s).appendResource("toto"));
            }
        } catch (MalformedPathException e) {
            Assert.assertTrue(false);
        }
    }

    /**
     * Test method for
     * {@link org.ow2.wildcat.hierarchy.Path#appendAttribute(java.lang.String)}.
     */
    @Test
    public void testAppendAttribute() {
        try {
            for (String s : this.resourcePaths) {
                Assert.assertNotNull(new Path(s).appendAttribute("toto"));
            }
            for (String s : this.attributePaths) {
                Assert.assertNull(new Path(s).appendAttribute("toto"));
            }
        } catch (MalformedPathException e) {
            Assert.assertTrue(false);
        }
    }

    /**
     * Test method for
     * {@link org.ow2.wildcat.hierarchy.Path#append(org.ow2.wildcat.hierarchy.Path)}.
     */
    @Test
    public void testAppend() {
        try {
            for (String suffix : this.relativePaths) {
                for (String s : this.resourcePaths) {
                    Assert.assertNotNull(new Path(s).append(new Path(suffix)));
                }
                for (String s : this.attributePaths) {
                    Assert.assertNull(new Path(s).append(new Path(suffix)));
                }
            }
        } catch (MalformedPathException e) {
            Assert.assertTrue(false);
        }
    }

    /**
     * Test method for {@link org.ow2.wildcat.hierarchy.Path#canonicalize()}.
     */
    @Test
    public void testCanonicalize() {
        Assert.assertEquals(this.noncanonicalize.length, this.canonicalize.length);
        for (int i = 0; i < this.noncanonicalize.length; i++) {
            try {
                Assert.assertEquals(new Path(this.noncanonicalize[i]).canonicalize()
                        .toString(), new Path(this.canonicalize[i]).toString());
            } catch (MalformedPathException e) {
                Assert.assertTrue(false);
            }
        }
    }

    /**
     * Test method for {@link org.ow2.wildcat.hierarchy.Path#stripTop()}.
     */
    @Test
    public void testStripTop() {
        for (String[] sa : this.stripTop) {
            try {
                Path p = new Path(sa[0]);
                p = p.stripTop();
                if (p == null) {
                    Assert.assertTrue(sa[1] == null);
                } else {
                    Assert.assertTrue(p.toString().equals(sa[1]));
                }
            } catch (MalformedPathException e) {
                Assert.assertTrue(false);
            }
        }
    }

    /**
     * Test method for {@link org.ow2.wildcat.hierarchy.Path#getRelativePart()}.
     *
     * @throws MalformedPathException
     */
    @Test
    public void testGetRelativePart() throws MalformedPathException {
        String[][] relativeParts = new String[][] {
                { "self://proc/cpu#load", "proc/cpu#load" },
                { "/proc/cpu#load", "proc/cpu#load" },
                { "proc/cpu#load", "proc/cpu#load" }, };
        for (String[] sa : relativeParts) {
            Assert.assertTrue(new Path(sa[0]).getRelativePart().toString().equals(
                    sa[1]));
        }
        Assert.assertNull(new Path("self://").getRelativePart());
    }

    /**
     * Test method for {@link org.ow2.wildcat.hierarchy.Path#getHostname()}.
     *
     * @throws MalformedPathException
     */
    @Test
    public void testGetHostname() throws MalformedPathException {
        String[][] hostnames = new String[][] {
                { "self://proc/cpu#load", "self" },
                { "local://proc/cpu#load", "local" },
                { "hello://#load", "hello" }, };
        for (String[] sa : hostnames) {
            Assert.assertTrue(new Path(sa[0]).getHostname().equals(sa[1]));
        }
        String[] nulls = new String[] { "/proc/cpu#load", "proc/cpu#load",
        "#load" };
        for (String n : nulls) {
            Assert.assertNull(new Path(n).getHostname());
        }
    }

    /**
     * Test method for {@link org.ow2.wildcat.hierarchy.Path#getAttributeName()}.
     *
     * @throws MalformedPathException
     */
    @Test
    public void testGetAttributeName() throws MalformedPathException {
        for (String s : this.attributePaths) {
            Assert.assertTrue("load".equals(new Path(s).getAttributeName()));
        }
        for (String s : this.resourcePaths) {
            Assert.assertNull(new Path(s).getAttributeName());
        }
    }

    /**
     * Test method for {@link org.ow2.wildcat.hierarchy.Path#getFirstResource()}.
     *
     * @throws MalformedPathException
     */
    @Test
    public void testGetFirstResource() throws MalformedPathException {
        String[] resources = new String[] { "self://proc/proc#proc",
                "/proc/proc#proc", "proc/proc#proc", "proc#proc", "proc", };
        for (String s : resources) {
            Assert.assertTrue("proc".equals(new Path(s).getFirstResource()));
        }
    }

    /**
     * Test method for {@link org.ow2.wildcat.hierarchy.Path#isAbsolute()}.
     *
     * @throws MalformedPathException
     */
    @Test
    public void testIsAbsolute() throws MalformedPathException {
        for (String s : this.absolutePaths) {
            Assert.assertTrue(new Path(s).isAbsolute());
        }
        for (String s : this.relativePaths) {
            Assert.assertFalse(new Path(s).isAbsolute());
        }
    }

    /**
     * Test method for {@link org.ow2.wildcat.hierarchy.Path#hasExplicitHost()}.
     *
     * @throws MalformedPathException
     */
    @Test
    public void testHasExplicitHost() throws MalformedPathException {
        String[] explicit = new String[] { "self://help#toto", "self://#toto",
                "self://toto", "self://", };
        for (String s : explicit) {
            Assert.assertTrue(new Path(s).hasExplicitHost());
        }
        String[] nonexplicit = new String[] { "/help#toto", "#toto", "/toto", };
        for (String s : nonexplicit) {
            Assert.assertFalse(new Path(s).hasExplicitHost());
        }
    }

    /**
     * Test method for {@link org.ow2.wildcat.hierarchy.Path#hasAttribute()}.
     *
     * @throws MalformedPathException
     */
    @Test
    public void testHasAttribute() throws MalformedPathException {
        for (String s : this.attributePaths) {
            Assert.assertTrue(new Path(s).hasAttribute());
        }
        for (String s : this.resourcePaths) {
            Assert.assertFalse(new Path(s).hasAttribute());
        }
    }

    /**
     * Test method for {@link org.ow2.wildcat.hierarchy.Path#hasResource()}.
     *
     * @throws MalformedPathException
     */
    @Test
    public void testHasResource() throws MalformedPathException {
        String[] withResources = new String[] { "self://proc",
                "self://proc/cpu", "self://proc/cpu#load", "/proc",
                "/proc/cpu", "/proc/cpu#load", "proc", "proc/cpu",
                "proc/cpu#load", };
        for (String s : withResources) {
            Assert.assertTrue(new Path(s).hasResource());
        }
        String[] withoutResources = new String[] { "#load", "/#load",
                "self://#load", };
        for (String s : withoutResources) {
            Assert.assertFalse(new Path(s).hasResource());
        }
    }

    /**
     * Test method for {@link org.ow2.wildcat.hierarchy.Path#isOnlyAttribute()}.
     *
     * @throws MalformedPathException
     */
    @Test
    public void testIsOnlyAttribute() throws MalformedPathException {
        String[] not = new String[] { "self://proc#toto", "self://#toto",
                "/#toto", };
        for (String s : not) {
            Assert.assertFalse(new Path(s).isOnlyAttribute());
        }
    }

    /**
     * Test method for {@link org.ow2.wildcat.hierarchy.Path#isOnlyResource()}.
     *
     * @throws MalformedPathException
     */
    @Test
    public void testIsOnlyResource() throws MalformedPathException {
        String[] not = new String[] { "self://proc", "proc/proc", "/proc", };
        for (String s : not) {
            Assert.assertFalse(new Path(s).isOnlyResource());
        }
    }

    /**
     * Test method for {@link org.ow2.wildcat.hierarchy.Path#isCanonical()}.
     *
     * @throws MalformedPathException
     */
    @Test
    public void testIsCanonical() throws MalformedPathException {
        String[] canonical = new String[] { "self://*/*#*",
                "self://proc/cpu#load", };
        for (String s : canonical) {
            Assert.assertTrue(new Path(s).isCanonical());
        }
        String[] noncanonical = new String[] { "self://proc/./cpu#load",
                "self://proc/../cpu#load", "self://*/./cpu", };
        for (String s : noncanonical) {
            Assert.assertFalse(new Path(s).isCanonical());
        }
    }

    /**
     * Test method for {@link org.ow2.wildcat.hierarchy.Path#isPattern()}.
     *
     * @throws MalformedPathException
     */
    @Test
    public void testIsPattern() throws MalformedPathException {
        String[] pattern = new String[] { "self://*/*#*", "self://proc/cpu#*",
                "self://proc/*#load", };
        for (String s : pattern) {
            Assert.assertTrue(new Path(s).isPattern());
        }
        String[] nonpattern = new String[] { "self://proc/cpu#load",
                "self://../proc/./cpu#load", };
        for (String s : nonpattern) {
            Assert.assertFalse(new Path(s).isPattern());
        }
    }

    /**
     * Test method for {@link org.ow2.wildcat.hierarchy.Path#toString()}.
     *
     * @throws MalformedPathException
     */
    @Test
    public void testToString() throws MalformedPathException {
        String[][] strs = new String[][] {
                { "self://proc/cpu#load", "self://proc/cpu#load" },
                { "/proc/cpu#load", "self://proc/cpu#load" },
                { "self://#load", "self://#load" },
                { "/#load", "self://#load" }, };
        for (String[] sa : strs) {
            Assert.assertTrue(sa[1].equals(new Path(sa[0]).toString()));
        }
    }

}
