/**
 * WildCAT: A Generic Framework for Context-Aware Applications.
 * Copyright (C) 2008-2009 Bull S.A.S.
 * Copyright (C) 2008 EMN
 * Contact: wildcat@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 * Authors:
 *		- Pierre-Charles David (initial)
 *		- Nicolas Loriant
 * --------------------------------------------------------------------------
 * $Id: Symlink.java 459 2010-01-04 10:41:22Z mahmoudbh $
 * --------------------------------------------------------------------------
 */
package org.ow2.wildcat.hierarchy.resource;

import java.text.MessageFormat;
import java.util.Set;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.ow2.wildcat.Context;
import org.ow2.wildcat.ContextException;
import org.ow2.wildcat.Query;
import org.ow2.wildcat.event.WEvent;
import org.ow2.wildcat.event.WEventInterface;
import org.ow2.wildcat.event.WHierarchyEvent;
import org.ow2.wildcat.event.WEventInterface.Scope;
import org.ow2.wildcat.hierarchy.MalformedPathException;
import org.ow2.wildcat.hierarchy.Mountable;
import org.ow2.wildcat.hierarchy.Path;
import org.ow2.wildcat.hierarchy.attribute.Attribute;
import org.ow2.wildcat.util.Messages;

import com.espertech.esper.client.UpdateListener;
import com.espertech.esper.client.EventBean;

/**
 * A resource that act as a symbolic link to a destination in the path.
 * It is the user responsibility not to create a cyclic hierarchy.
 */
public class Symlink implements Resource, UpdateListener {

    /**
     * Logger.
     */
    private static Log logger = LogFactory.getLog(Symlink.class);

    private static String EVT_REROUTE = "select * from WEvent (org.ow2.wildcat.util.Helper.isPrefix(source, \"{0}\"))";

    /**
     * local context
     */
    private Context context;

    /**
     * Mount point
     */
    private Path path;

    /**
     * Destination resource
     */
    private final Path target;

    /**
     * Query dealing with intercepting events.
     */
    private Query query;

    /**
     * Constructor.
     *
     * @param target
     *            destination of the symbolic link
     */
    public Symlink(final Path target) {
        this.target = target;
    }

    private Path addPrefix(final Path suffix) {
        return this.target.append(suffix);
    }

    /*
     * (non-Javadoc)
     *
     * @see org.ow2.wildcat.hierarchy.resource.Resource#attachAttribute(org.ow2.wildcat.hierarchy.Path,
     *      org.ow2.wildcat.hierarchy.attribute.Attribute)
     */
    public void attachAttribute(final Path attribute, final Attribute attr) throws ResourceException {
        try {
            this.context.attachAttribute(
                    this.addPrefix(attribute).toString(), attr);
        } catch (ContextException e) {
            logger.error("Unable to attach an attribute at path " + path, e);
            throw new ResourceException("Unable to attach an attribute at path " + path, e);
        }
    }

    /*
     * (non-Javadoc)
     *
     * @see org.ow2.wildcat.hierarchy.resource.Resource#attachResource(org.ow2.wildcat.hierarchy.Path,
     *      org.ow2.wildcat.hierarchy.resource.Resource)
     */
    public void attachResource(final Path resource, final Resource res) throws ResourceException {
        try {
            this.context.attachResource(
                    this.addPrefix(resource).toString(), res);
        } catch (ContextException e) {
            logger.error("Unable to attach a resource at path " + path, e);
            throw new ResourceException("Unable to attach a resource at path " + path, e);
        }
    }

    /*
     * (non-Javadoc)
     *
     * @see org.ow2.wildcat.hierarchy.resource.Resource#attachAttribute(org.ow2.wildcat.hierarchy.Path,
     *      org.ow2.wildcat.hierarchy.attribute.Attribute)
     */
    public void createAttribute(
            final Path attribute, final WEventInterface.Scope scope, final Object value) throws ResourceException {
        try {
            this.context.createAttribute(
                    this.addPrefix(attribute).toString(), scope, value);
        } catch (ContextException e) {
            logger.error("Unable to create an attribute at path " + path, e);
            throw new ResourceException("Unable to create an attribute at path " + path, e);
        }
    }

    /**
     * @param path
     * @return
     * @throws ResourceException
     */
    public Attribute getAttribute(final Path attribute) throws ResourceException {
        try {
            return this.context.getAttribute(
                    this.addPrefix(attribute).toString());
        } catch (ContextException e) {
            logger.error("Unable to get an attribute at path " + path, e);
            throw new ResourceException("Unable get an attribute at path " + path, e);
        }
    }

    /**
     * @param attribute
     * @throws ResourceException
     */
    public void detachAttribute(final Path attribute) throws ResourceException {
        try {
            context.detachAttribute(this.addPrefix(attribute).toString());
        } catch (ContextException e) {
            logger.error("Unable to detach an attribute at path " + path, e);
            throw new ResourceException("Unable detach an attribute at path " + path, e);
        }
    }

    /*
     * (non-Javadoc)
     *
     * @see org.ow2.wildcat.hierarchy.resource.Resource#createResource(org.ow2.wildcat.hierarchy.Path)
     */
    public void createResource(final Path resource) throws ResourceException {
        try {
            this.context.createResource(
                    this.addPrefix(resource).toString());
        } catch (ContextException e) {
            logger.error("Unable to create a resource at path " + path, e);
            throw new ResourceException("Unable to create a resource at path " + path, e);
        }
    }

    /**
     * @param resource
     * @throws ResourceException
     */
    public void detachResource(final Path resource) throws ResourceException {
        try {
            context.detachAttribute(this.addPrefix(resource).toString());
        } catch (ContextException e) {
            logger.error("Unable to detach a resource at path " + path, e);
            throw new ResourceException("Unable detach a resource at path " + path, e);
        }
    }

    /**
     * @param res
     * @return
     * @throws ResourceException
     */
    public Resource getResource(final Path res) throws ResourceException {
        try {
            return context.getResource(addPrefix(res).toString());
        } catch (ContextException e) {
            logger.error("Unable to get a resource at path " + path, e);
            throw new ResourceException("Unable to get a resource at path " + path, e);
        }
    }

    /*
     * (non-Javadoc)
     *
     * @see org.ow2.wildcat.hierarchy.resource.Resource#getValue(org.ow2.wildcat.hierarchy.Path)
     */
    public Object getValue(final Path path) throws ResourceException {
        try {
            return this.context.getValue(
                    this.addPrefix(path).toString());
        } catch (ContextException e) {
            logger.error("Unable to get a value at path " + path, e);
            throw new ResourceException("Unable to get a value at path " + path, e);
        }
    }

    public final Path getTarget() {
        return target;
    }

    /*
     * (non-Javadoc)
     *
     * @see org.ow2.wildcat.hierarchy.resource.Resource#list(org.ow2.wildcat.hierarchy.Path)
     */
    public Set<String> list(final Path resource) throws ResourceException {
        try {
            return this.context.list(
                    this.addPrefix(resource).toString());
        } catch (ContextException e) {
            logger.error("Unable to list a resource at path " + path, e);
            throw new ResourceException("Unable to list a resource at path " + path, e);
        }
    }

    /*
     * (non-Javadoc)
     *
     * @see org.ow2.wildcat.hierarchy.resource.Resource#setValue(org.ow2.wildcat.hierarchy.Path,
     *      java.lang.Object)
     */
    public Object setValue(final Path path, final Object value) throws ResourceException {
        try {
            return this.context.setValue(
                    this.addPrefix(path).toString(), value);
        } catch (ContextException e) {
            logger.error("Unable to set a value at path " + path, e);
            throw new ResourceException("Unable to set a value at path " + path, e);
        }
    }

    /*
     * (non-Javadoc)
     *
     * @see org.ow2.wildcat.hierarchy.Mountable#isMount()
     */
    public boolean isMount() {
        return context != null && path != null;
    }

    /*
     * (non-Javadoc)
     *
     * @see org.ow2.wildcat.hierarchy.Mountable#mount(org.ow2.wildcat.Context,
     *      org.ow2.wildcat.hierarchy.Path)
     */
    public void mount(final Context context, final Path mountPoint) {
        logger.debug(Messages.message(Mountable.class.getName() + ".mount",
                mountPoint));

        if (this.isMount()) {
            logger.info(Messages.message(Mountable.class.getName()
                    + ".alreadymount", mountPoint));
        } else {
            this.context = context;
            this.path = mountPoint;
            this.context.emitEvent(new WHierarchyEvent(this.path,
                    WHierarchyEvent.Type.LINKED, this.target.toString()));

            this.query = this.context.registerListeners(MessageFormat.format(
                    EVT_REROUTE, this.target.toString()), this);
        }
    }

    /*
     * (non-Javadoc)
     *
     * @see org.ow2.wildcat.hierarchy.Mountable#unMount()
     */
    public void unMount() {
        logger.debug(Messages.message(Mountable.class.getName() + ".unmount",
                (Object[]) null));

        if (!this.isMount()) {
            logger.info(Messages.message(Mountable.class.getName()
                    + ".notmount", (Object[]) null));
        } else {
            this.context.emitEvent(new WHierarchyEvent(this.path,
                    WHierarchyEvent.Type.UNLINKED, this.target.toString()));
            if (this.query != null) {
                this.context.destroyQuery(this.query);
                this.query = null;
            }
            context = null;
            path = null;
        }
    }

    /*
     * (non-Javadoc)
     *
     * @see net.esper.client.UpdateListener#update(net.esper.event.EventBean[],
     *      net.esper.event.EventBean[])
     */
    public void update(final EventBean[] arg0, final EventBean[] arg1) {
        for (EventBean bean : arg0) {
            if (bean.getUnderlying() instanceof WEvent) {
                try {
                    WEvent evt = (WEvent) ((WEvent) bean.getUnderlying())
                    .clone();
                    Path s = new Path(evt.getSource());
                    evt.setSource(s.replacePrefix(this.target, this.path));

                    // Use a local scope to avoid infinite loops
                    evt.setScope(Scope.LOCAL);

                    if (this.isMount()) {
                        this.context.emitEvent(evt);
                    }
                } catch (CloneNotSupportedException e) {
                    logger.fatal("REALLY BIG TROUBLE", e);
                } catch (MalformedPathException e) {
                    logger.fatal("REALLY BIG TROUBLE", e);
                }
            }
        }
    }

}
